using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Android.App;
using Android.Content;
using Android.OS;
using Android.Runtime;
using Android.Views;
using Android.Widget;

using Com.Syncfusion.SfPicker;
using Native = Com.Syncfusion.SfPicker.SfPicker;


using Syncfusion.SfPicker.XForms;
using Xamarin.Forms;
using Xamarin.Forms.Platform.Android;
using System.Collections.Specialized;
using System.Collections.ObjectModel;
using Android.Util;
using Android.Content.Res;
using Android.Graphics;

namespace Syncfusion.SfPicker.XForms.Droid
{
	/// <summary>
	/// Sf busy indicator mapping.
	/// </summary>
    public class SfPickerMapping
    {
        #region Properties
		/// <summary>
		/// The busy indicator.
		/// </summary>
        Native Picker;
		SfPicker formsPicker;
		internal static bool isSelectionChange = true;

		#endregion

		#region Methods
		/// <summary>
		/// Raises the busy indicator properties changed event.
		/// </summary>
		/// <param name="propertyName">Property name.</param>
		/// <param name="formsRotator">Forms busy indicator.</param>
		/// <param name="nativeRotator">Native busy indicator.</param>
		public void OnPickerPropertiesChanged(string propertyName, SfPicker formsPicker, Native nativePicker)
        {
			this.formsPicker = formsPicker;
			Picker = nativePicker;
			if (propertyName == SfPicker.SelectedItemTextColorProperty.PropertyName)
			{
				Picker.SelectedItemTextcolor = new global::Android.Graphics.Color()
				{
					A = (byte)(formsPicker.SelectedItemTextColor.A * 255),
					R = (byte)(formsPicker.SelectedItemTextColor.R * 255),
					G = (byte)(formsPicker.SelectedItemTextColor.G * 255),
					B = (byte)(formsPicker.SelectedItemTextColor.B * 255)
				};
			}
			else if (propertyName == SfPicker.UnSelectedItemTextColorProperty.PropertyName)
			{
				Picker.UnSelectedItemTextColor = new global::Android.Graphics.Color()
				{
					A = (byte)(formsPicker.UnSelectedItemTextColor.A * 255),
					R = (byte)(formsPicker.UnSelectedItemTextColor.R * 255),
					G = (byte)(formsPicker.UnSelectedItemTextColor.G * 255),
					B = (byte)(formsPicker.UnSelectedItemTextColor.B * 255)
				};
			}
			else if (propertyName == SfPicker.BorderColorProperty.PropertyName)
			{
				Picker.BorderColor = new global::Android.Graphics.Color()
				{
					A = (byte)(formsPicker.BorderColor.A * 255),
					R = (byte)(formsPicker.BorderColor.R * 255),
					G = (byte)(formsPicker.BorderColor.G * 255),
					B = (byte)(formsPicker.BorderColor.B * 255)
				};
			}
			else if (propertyName == SfPicker.BackgroundColorProperty.PropertyName)
			{
				Picker.BackgroundColor = new global::Android.Graphics.Color()
				{
					A = (byte)(formsPicker.BackgroundColor.A * 255),
					R = (byte)(formsPicker.BackgroundColor.R * 255),
					G = (byte)(formsPicker.BackgroundColor.G * 255),
					B = (byte)(formsPicker.BackgroundColor.B * 255)
				};
			}
			else if (propertyName == SfPicker.SelectedItemProperty.PropertyName)
			{
                if (isSelectionChange)
                {
                    if (formsPicker.SelectedItem != null && formsPicker.SelectedItem is ObservableCollection<object>)
                    {
                        ObservableCollection<object> tempSource = new ObservableCollection<object>();
                        foreach (var item in formsPicker.SelectedItem as ObservableCollection<object>)
                        {
                            tempSource.Add(item);
                        }
                        Picker.SelectedItem = tempSource;
                    }
                    else if (formsPicker.SelectedItem is Object)
                    {
                        ObservableCollection<object> tempSource = new ObservableCollection<object>();
                        tempSource.Add(formsPicker.SelectedItem);
                        Picker.SelectedItem = tempSource;
                    }
                }

			}
			else if (propertyName == SfPicker.SelectedIndexProperty.PropertyName)
			{
                if (isSelectionChange)
                {
                    if (formsPicker.SelectedIndex != null && formsPicker.SelectedIndex is ObservableCollection<int>)
                    {
                        ObservableCollection<int> tempSource = formsPicker.SelectedIndex as ObservableCollection<int>;
                        Picker.SelectedIndex = tempSource;
                    }
                    else if (formsPicker.SelectedIndex is Object)
                    {
                        ObservableCollection<object> tempSource = new ObservableCollection<object>();
                        tempSource.Add(formsPicker.SelectedIndex);
                        if(Picker.SelectedIndex!=tempSource)
                        Picker.SelectedIndex = tempSource;
                    }
                }

			}
			else if (propertyName == SfPicker.HeaderFontSizeProperty.PropertyName)
			{
				Picker.HeaderTextSize = formsPicker.HeaderFontSize;
			}
            else if (propertyName == SfPicker.ColumnHeaderFontSizeProperty.PropertyName)
            {
                Picker.ColumnHeaderTextSize = formsPicker.ColumnHeaderFontSize;
            }
			else if (propertyName == SfPicker.ItemsSourceProperty.PropertyName)
			{
				ObservableCollection<object> tempSource = new ObservableCollection<object>();
				if (formsPicker.ItemsSource != null)
				{
					foreach (var item in formsPicker.ItemsSource)
					{
						tempSource.Add(item);
					}
                    if ((formsPicker.ItemsSource as INotifyCollectionChanged) != null)
						(formsPicker.ItemsSource as INotifyCollectionChanged).CollectionChanged += Handle_CollectionChanged;
				}
				Picker.ItemsSource = tempSource;
			}
			else if (propertyName == SfPicker.ColumnHeaderTextProperty.PropertyName)
			{
				ObservableCollection<string> tempSource = new ObservableCollection<string>();
				if (formsPicker.ColumnHeaderText != null)
				{
					if (formsPicker.ColumnHeaderText is ObservableCollection<string> )
					{
						foreach (var item in formsPicker.ColumnHeaderText as ObservableCollection<string>)
						{
							tempSource.Add(item.ToString());
						}
					}
                    else if (formsPicker.ColumnHeaderText is string)
                    {
                        tempSource.Add(formsPicker.ColumnHeaderText.ToString());
                    }
				}
				Picker.ColumnHeaderText =tempSource;
			}
			else if (propertyName == SfPicker.HeaderHeightProperty.PropertyName)
			{
				Picker.HeaderHeight = formsPicker.HeaderHeight * Forms.Context.Resources.DisplayMetrics.Density;
			}
			else if (propertyName == SfPicker.FooterHeightProperty.PropertyName)
			{
				Picker.FooterHeight = formsPicker.FooterHeight * Forms.Context.Resources.DisplayMetrics.Density;
			}
			else if (propertyName == SfPicker.PickerHeightProperty.PropertyName)
			{
				Picker.PickerHeight = formsPicker.PickerHeight * Forms.Context.Resources.DisplayMetrics.Density;
				formsPicker.HeightRequest = formsPicker.PickerHeight;
			}
			else if (propertyName == SfPicker.PickerWidthProperty.PropertyName)
			{
				Picker.PickerWidth = formsPicker.PickerWidth * Forms.Context.Resources.DisplayMetrics.Density;
				formsPicker.WidthRequest = formsPicker.PickerWidth;
			}
			else if (propertyName == SfPicker.ItemsHeightProperty.PropertyName)
			{
				Picker.ItemHeight = formsPicker.ItemHeight * Forms.Context.Resources.DisplayMetrics.Density; ;
			}
			else if (propertyName == SfPicker.ColumnHeaderHeightProperty.PropertyName)
			{
				Picker.ColumnHeaderHeight = formsPicker.ColumnHeaderHeight * Forms.Context.Resources.DisplayMetrics.Density; ;
			}
			else if (propertyName == SfPicker.PickerModeProperty.PropertyName)
			{
				if (formsPicker.PickerMode == PickerMode.Default)
					Picker.PickerMode = Com.Syncfusion.SfPicker.PickerMode.Default;
				else
					Picker.PickerMode = Com.Syncfusion.SfPicker.PickerMode.Dialog;
			}
			else if (propertyName == SfPicker.IsOpenProperty.PropertyName)
			{
				if (Picker.IsOpen != formsPicker.IsOpen)
					Picker.IsOpen = formsPicker.IsOpen;
			}
			else if (propertyName == SfPicker.ShowHeaderProperty.PropertyName)
			{
				Picker.ShowHeader = formsPicker.ShowHeader;
			}
			else if (propertyName == SfPicker.ShowFooterProperty.PropertyName)
			{
				Picker.ShowFooter = formsPicker.ShowFooter;
			}
			else if (propertyName == SfPicker.ShowColumnHeaderProperty.PropertyName)
			{
				Picker.ShowColumnHeader = formsPicker.ShowColumnHeader;
			}
			else if (propertyName == SfPicker.HeaderTextProperty.PropertyName)
			{
				Picker.HeaderText = formsPicker.HeaderText;
			}
			else if (propertyName == SfPicker.HeaderViewProperty.PropertyName)
			{
				if (formsPicker.HeaderView != null)
					Picker.HeaderView = SfPickerRenderer.ConvertFormsToNative(formsPicker.HeaderView, new Rectangle(0, 0, 100, 100), formsPicker);
			}
			else if (propertyName == SfPicker.FooterViewProperty.PropertyName)
			{
				if (formsPicker.FooterView != null)
					Picker.FooterView = SfPickerRenderer.ConvertFormsToNative(formsPicker.HeaderView, new Rectangle(0, 0, 100, 100), formsPicker);
			}
			else if (propertyName == SfPicker.HeaderFontAttributeProperty.PropertyName)
			{
				if (formsPicker.HeaderFontAttribute != null)
					nativePicker.HeaderTypeface = GetNativeFont(formsPicker.HeaderFontAttribute, formsPicker.HeaderFontFamily);
			}

			else if (propertyName == SfPicker.ColumnHeaderFontAttributeProperty.PropertyName)
			{
				if (formsPicker.ColumnHeaderFontAttribute != null)
					nativePicker.ColumnHeaderTypeface = GetNativeFont(formsPicker.ColumnHeaderFontAttribute, formsPicker.ColumnHeaderFontFamily);
			}
			else if (propertyName == SfPicker.UnSelectedItemsFontAttributeProperty.PropertyName)
			{
				if (formsPicker.UnSelectedItemFontAttribute != null)
					nativePicker.UnSelectedItemTypeface = GetNativeFont(formsPicker.UnSelectedItemFontAttribute, formsPicker.UnSelectedItemFontFamily);
			}
			else if (propertyName == SfPicker.SelectedItemFontAttributeProperty.PropertyName)
			{
				if (formsPicker.SelectedItemFontAttribute != null)
					nativePicker.SelectedItemTypeface = GetNativeFont(formsPicker.SelectedItemFontAttribute, formsPicker.SelectedItemFontFamily);
			}
			else if (propertyName == SfPicker.HeaderFontFamilyProperty.PropertyName)
			{
				if (formsPicker.HeaderFontAttribute != null)
				{
					nativePicker.HeaderTypeface = GetNativeFont(formsPicker.HeaderFontAttribute, formsPicker.HeaderFontFamily);
				}
			}
			else if (propertyName == SfPicker.ColumnHeaderFontFamilyProperty.PropertyName)
			{
				if (formsPicker.ColumnHeaderFontAttribute != null)
				{
					nativePicker.ColumnHeaderTypeface = GetNativeFont(formsPicker.ColumnHeaderFontAttribute, formsPicker.ColumnHeaderFontFamily);
				}
			}
			else if (propertyName == SfPicker.UnSelectedItemsFontSizeProperty.PropertyName)
			{
				nativePicker.UnSelectedItemTextSize = formsPicker.UnSelectedItemFontSize* Forms.Context.Resources.DisplayMetrics.Density;
			}
			else if (propertyName == SfPicker.SelectedItemFontSizeProperty.PropertyName)
			{
				nativePicker.SelectedItemTextSize = formsPicker.SelectedItemFontSize* Forms.Context.Resources.DisplayMetrics.Density; 
			}
			else if (propertyName == SfPicker.HeaderBackgroundColorProperty.PropertyName)
			{
				nativePicker.HeaderBackgroundColor = new global::Android.Graphics.Color()
				{
					A = (byte)(formsPicker.HeaderBackgroundColor.A * 255),
					R = (byte)(formsPicker.HeaderBackgroundColor.R * 255),
					G = (byte)(formsPicker.HeaderBackgroundColor.G * 255),
					B = (byte)(formsPicker.HeaderBackgroundColor.B * 255)
				};
			}
			else if (propertyName == SfPicker.HeaderBackgroundColorProperty.PropertyName)
			{
				nativePicker.HeaderBackgroundColor = new global::Android.Graphics.Color()
				{
					A = (byte)(formsPicker.HeaderBackgroundColor.A * 255),
					R = (byte)(formsPicker.HeaderBackgroundColor.R * 255),
					G = (byte)(formsPicker.HeaderBackgroundColor.G * 255),
					B = (byte)(formsPicker.HeaderBackgroundColor.B * 255)
				};
			}
			else if (propertyName == SfPicker.ColumnHeaderBackgroundColorProperty.PropertyName)
			{
				nativePicker.ColumnHeaderBackgroundColor = new global::Android.Graphics.Color()
				{
					A = (byte)(formsPicker.ColumnHeaderBackgroundColor.A * 255),
					R = (byte)(formsPicker.ColumnHeaderBackgroundColor.R * 255),
					G = (byte)(formsPicker.ColumnHeaderBackgroundColor.G * 255),
					B = (byte)(formsPicker.ColumnHeaderBackgroundColor.B * 255)
				};
			}
			else if (propertyName == SfPicker.HeaderTextColorProperty.PropertyName)
			{
				nativePicker.HeaderTextColor = new global::Android.Graphics.Color()
				{
					A = (byte)(formsPicker.HeaderTextColor.A * 255),
					R = (byte)(formsPicker.HeaderTextColor.R * 255),
					G = (byte)(formsPicker.HeaderTextColor.G * 255),
					B = (byte)(formsPicker.HeaderTextColor.B * 255)
				};
			}
			else if (propertyName == SfPicker.ColumnHeaderTextColorProperty.PropertyName)
			{
				nativePicker.ColumnHeaderTextColor = new global::Android.Graphics.Color()
				{
					A = (byte)(formsPicker.ColumnHeaderTextColor.A * 255),
					R = (byte)(formsPicker.ColumnHeaderTextColor.R * 255),
					G = (byte)(formsPicker.ColumnHeaderTextColor.G * 255),
					B = (byte)(formsPicker.ColumnHeaderTextColor.B * 255)
				};
			}

        }

		void Handle_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
		{
			ObservableCollection<object> tempSource = new ObservableCollection<object>();
			if (formsPicker.ItemsSource != null)
			{
				foreach (var item in formsPicker.ItemsSource)
				{
					tempSource.Add(item);
				}
			}
			Picker.ItemsSource = tempSource;
		}

		public static Typeface GetNativeFont(FontAttributes font, string fontFamily)
		{

			if (font == FontAttributes.Bold)
			{
				return Typeface.Create(fontFamily, TypefaceStyle.Bold);
			}
			else if (font == FontAttributes.Italic)
			{
				return Typeface.Create(fontFamily, TypefaceStyle.Italic);
			}
			else if (font == FontAttributes.None)
			{
				return Typeface.Create(fontFamily, TypefaceStyle.Normal);
			}

			return Typeface.Default;
		}

        #endregion
    }


}