using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Android.App;
using Android.Content;
using Android.OS;
using Android.Runtime;
using Android.Views;
using Android.Widget;
using Xamarin.Forms;
using Native = Com.Syncfusion.SfPicker.SfPicker;

using Xamarin.Forms.Platform.Android;
using System.Reflection;
using Syncfusion.SfPicker.XForms;
using Syncfusion.SfPicker.XForms.Droid;
using System.Collections.ObjectModel;

[assembly: ExportRenderer(typeof(Syncfusion.SfPicker.XForms.SfPicker), typeof(Syncfusion.SfPicker.XForms.Droid.SfPickerRenderer))]

namespace Syncfusion.SfPicker.XForms.Droid
{
    [Preserve(AllMembers = true)]
	/// <summary>
	/// Sf busy indicator renderer.
	/// </summary>
	public class SfPickerRenderer : ViewRenderer<Syncfusion.SfPicker.XForms.SfPicker, Native>
	{
		/// <summary>
		/// The busy indicator.
		/// </summary>
		SfPicker picker;

		int prevValue = 0;
		/// <summary>
		/// The native busy indicator.
		/// </summary>
		Native nativePicker;
		/// <summary>
		/// Initializes a new instance of the <see cref="Syncfusion.SfPicker.XForms.Droid.SfPickerRenderer"/> class.
		/// </summary>
		public SfPickerRenderer()
		{

		}
		/// <summary>
		/// The sf busy indicator mapping.
		/// </summary>
		SfPickerMapping SfPickerMapping = new SfPickerMapping();
		/// <summary>
		/// Raises the element changed event.
		/// </summary>
		/// <param name="e">E.</param>
		protected override void OnElementChanged(ElementChangedEventArgs<Syncfusion.SfPicker.XForms.SfPicker> e)
		{
			base.OnElementChanged(e);
			if (e.NewElement != null)
			{
				picker = e.NewElement;
				nativePicker = new Native(Forms.Context);
				List<string> propertiesNames = GetPropertiesChanged(typeof(SfPicker), picker);
				foreach (var name in propertiesNames)
				{
					SfPickerMapping.OnPickerPropertiesChanged(name, picker, nativePicker);
				}
				nativePicker.OnSelectionChanged += NativePicker_ValueChanged;
				nativePicker.OnColumnLoaded += NativePicker_OnColumnHeaderViewLoaded;
				nativePicker.OnPickerItemLoaded += NativePicker_OnPickerItemLoaded;
				nativePicker.OkButtonClicked += NativePicker_OkButtonClicked;
				nativePicker.CancelButtonClicked += NativePicker_CancelButtonClicked;
				nativePicker.Opened += NativePicker_PopupOpened;
				nativePicker.Closed += NativePicker_PopupClosed;
				PickerAndroid.nativePicker = nativePicker;
				SetNativeControl(nativePicker);
				propertiesNames.Clear();
			}
		}

		protected override void OnElementPropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
		{
			if (sender is SfPicker)
			{
				SfPickerMapping = new SfPickerMapping();
				SfPickerMapping.OnPickerPropertiesChanged(e.PropertyName, sender as SfPicker, nativePicker);
			}
		}
		Color tempColor;bool isColorUpdated = false;
		void NativePicker_PopupOpened (object sender,EventArgs e)
		{
			EventArgs args = new EventArgs();
			SfPickerRenderer.TriggerInternalMethod(typeof(SfPicker), picker, "OnOpened", args);
			picker.IsOpen = true;
			if (picker.Parent != null)
			{
				tempColor = picker.Parent.BackgroundColor;
				isColorUpdated = true;
				picker.Parent.BackgroundColor = Color.FromRgba(128, 128, 128, 0.5);

			}
		}

		void NativePicker_PopupClosed (object sender, EventArgs e)
		{
			EventArgs args = new EventArgs();
			SfPickerRenderer.TriggerInternalMethod(typeof(SfPicker), picker, "OnClosed", args);
			picker.IsOpen = false;
			if (isColorUpdated && picker.Parent != null)
			{
				picker.Parent.BackgroundColor = tempColor;
				picker.Parent.Opacity = 1;
			}
		}

		void NativePicker_ValueChanged(object sender, Com.Syncfusion.SfPicker.SelectionChangedEventArgs e)
		{
			SfPickerMapping.isSelectionChange = false;
			SelectionChangedEventArgs args = new SelectionChangedEventArgs();
            picker.SelectedItem = e.NewValue;
            if(picker.SelectedIndex is ObservableCollection<object>)
                picker.SelectedIndex = nativePicker.SelectedIndex;
            else
            {
                if (nativePicker.SelectedIndex != null && nativePicker.SelectedIndex is ObservableCollection<object> && (nativePicker.SelectedIndex as ObservableCollection<object>).Count > 0)
                {
                    if(picker.SelectedIndex != (nativePicker.SelectedIndex as ObservableCollection<object>)[0])
                    picker.SelectedIndex = (nativePicker.SelectedIndex as ObservableCollection<object>)[0];
                }
                else if (nativePicker.SelectedIndex is Object && picker.SelectedIndex!=nativePicker.SelectedIndex)
                    picker.SelectedIndex = nativePicker.SelectedIndex;
            }
			SfPickerRenderer.SetInternalProperty(typeof(SelectionChangedEventArgs), args, "NewValue", e.NewValue);
			SfPickerRenderer.SetInternalProperty(typeof(SelectionChangedEventArgs), args, "OldValue", e.OldValue);
            SfPickerMapping.isSelectionChange = true;
			SfPickerRenderer.TriggerInternalMethod(typeof(SfPicker), picker, "OnSelectionChanged", args);

		}

		void NativePicker_OnColumnHeaderViewLoaded (object sender, Com.Syncfusion.SfPicker.ColumnLoadedEventArgs e)
		{
			ColumnLoadedEventArgs args = new ColumnLoadedEventArgs();
			args.ColumnWidth = e.ColumnWidth;
			SfPickerRenderer.SetInternalProperty(typeof(ColumnLoadedEventArgs), args, "Column", e.Coulmn);
			SfPickerRenderer.TriggerInternalMethod(typeof(SfPicker), picker, "ColumnLoaded", args);
			if(args.View != null)
				e.CustomView = ConvertFormsToNative(args.View, new Rectangle(0, 0, 100, 100), picker);
			e.ColumnWidth = args.ColumnWidth;
		}

		void NativePicker_OnPickerItemLoaded(object sender, Com.Syncfusion.SfPicker.PickerViewEventsArgs e)
		{
			PickerViewEventArgs args = new PickerViewEventArgs();
			SfPickerRenderer.SetInternalProperty(typeof(PickerViewEventArgs), args, "Column", e.Column);
			SfPickerRenderer.SetInternalProperty(typeof(PickerViewEventArgs), args, "Row", e.Row);
			SfPickerRenderer.SetInternalProperty(typeof(PickerViewEventArgs), args, "Item", e.Item);
			if (picker.ItemTemplate == null)
				SfPickerRenderer.TriggerInternalMethod(typeof(SfPicker), picker, "OnGetView", args);
			else
			{
				e.CustomView = ConvertFormsTemplateToNativeView(picker.ItemTemplate,args.Row,args.Column, new Rectangle(0, 0, 100, 100), picker);
			}

			if (args.View != null)
				e.CustomView = ConvertFormsToNative(args.View, new Rectangle(0, 0, 100, 100), picker);
		}

		void NativePicker_OkButtonClicked (object sender, Com.Syncfusion.SfPicker.SelectionChangedEventArgs e)
		{
			SelectionChangedEventArgs args = new SelectionChangedEventArgs();
			SfPickerRenderer.SetInternalProperty(typeof(SelectionChangedEventArgs), args, "NewValue", e.NewValue);
			SfPickerRenderer.SetInternalProperty(typeof(SelectionChangedEventArgs), args, "OldValue", e.OldValue);
			SfPickerRenderer.TriggerInternalMethod(typeof(SfPicker), picker, "OnOkButtonClicked", args);
			picker.IsOpen = false;
			if (isColorUpdated && picker.Parent != null)
			{
				picker.Parent.BackgroundColor = tempColor;
				picker.Parent.Opacity = 1;
			}
		}

        void NativePicker_CancelButtonClicked (object sender, Com.Syncfusion.SfPicker.SelectionChangedEventArgs e)
		{
			SelectionChangedEventArgs args = new SelectionChangedEventArgs();
			SfPickerRenderer.SetInternalProperty(typeof(SelectionChangedEventArgs), args, "NewValue", e.NewValue);
			SfPickerRenderer.SetInternalProperty(typeof(SelectionChangedEventArgs), args, "OldValue", e.OldValue);
			SfPickerRenderer.TriggerInternalMethod(typeof(SfPicker), picker, "OnCancelButtonClicked", args);
			picker.IsOpen = false;
			if (isColorUpdated && picker.Parent != null)
			{
				picker.Parent.BackgroundColor = tempColor;
				picker.Parent.Opacity = 1;
			}
		}





		/// <summary>
		/// Gets the properties changed.
		/// </summary>
		/// <returns>The properties changed.</returns>
		/// <param name="type">Type.</param>
		/// <param name="obj">Object.</param>
		public static List<string> GetPropertiesChanged(Type type, Object obj)
		{
			var properties = type.GetProperties();
			List<string> propertiesList = new List<string>();
			foreach (var item in properties)
			{
				propertiesList.Add(item.Name);
			}
			return propertiesList;
		}
		internal static object TriggerInternalMethod(Type type, object obj, string methodName, params object[] args)
		{
			try
			{
				var method = type.GetMethod(methodName, BindingFlags.NonPublic | BindingFlags.Instance);
				return method != null ? method.Invoke(obj, args) : null;
			}
			catch (Exception ex)
			{
				return ex;
			}
		}

		internal static void SetInternalProperty(Type type, object obj, string propertyName, object value)
		{
			var property = type.GetProperty(propertyName);

			if (property != null)
			{
				property.SetValue(obj, value);
			}
		}

		/// <summary>
		/// Gets the RG.
		/// </summary>
		/// <returns>The RG.</returns>
		/// <param name="hue">Hue.</param>
		/// <param name="saturation">Saturation.</param>
		/// <param name="luminosity">Luminosity.</param>
		public static Android.Graphics.Color GetRGB(float hue, float saturation, float luminosity)
		{
			int r = 0, g = 0, b = 0;
			if (saturation == 0)
			{
				r = g = b = (int)(luminosity * 255.0f + 0.5f);
			}
			else
			{
				float h = (hue - (float)Math.Floor(hue)) * 6.0f;
				float f = h - (float)Math.Floor(h);
				float p = luminosity * (1.0f - saturation);
				float q = luminosity * (1.0f - saturation * f);
				float t = luminosity * (1.0f - (saturation * (1.0f - f)));
				switch ((int)h)
				{
				case 0:
					r = (int)(luminosity * 255.0f + 0.5f);
					g = (int)(t * 255.0f + 0.5f);
					b = (int)(p * 255.0f + 0.5f);
					break;
				case 1:
					r = (int)(q * 255.0f + 0.5f);
					g = (int)(luminosity * 255.0f + 0.5f);
					b = (int)(p * 255.0f + 0.5f);
					break;
				case 2:
					r = (int)(p * 255.0f + 0.5f);
					g = (int)(luminosity * 255.0f + 0.5f);
					b = (int)(t * 255.0f + 0.5f);
					break;
				case 3:
					r = (int)(p * 255.0f + 0.5f);
					g = (int)(q * 255.0f + 0.5f);
					b = (int)(luminosity * 255.0f + 0.5f);
					break;
				case 4:
					r = (int)(t * 255.0f + 0.5f);
					g = (int)(p * 255.0f + 0.5f);
					b = (int)(luminosity * 255.0f + 0.5f);
					break;
				case 5:
					r = (int)(luminosity * 255.0f + 0.5f);
					g = (int)(p * 255.0f + 0.5f);
					b = (int)(q * 255.0f + 0.5f);
					break;
				}
			}
			return new Android.Graphics.Color(r,g,b); 
		}
		/// <summary>
		/// 
		/// </summary>
		/// <param name="view"></param>
		/// <param name="size"></param>
		/// <returns></returns>
		internal static ViewGroup ConvertFormsToNative(Xamarin.Forms.View view, Rectangle size, SfPicker formsRotator)
		{
			view.InputTransparent = false;
			var viewsize = view.GetSizeRequest(double.PositiveInfinity, double.PositiveInfinity);
			view.Layout(new Rectangle(0, 0, (int)size.Width, (int)size.Height));
			var vRenderer = Convert(view, (Xamarin.Forms.VisualElement)formsRotator.ParentView);
			var viewGroup = vRenderer as Android.Views.ViewGroup;
			vRenderer.Tracker.UpdateLayout();
			var layoutParams = new ViewGroup.LayoutParams(Android.Views.ViewGroup.LayoutParams.MatchParent, Android.Views.ViewGroup.LayoutParams.MatchParent);
			removeItemFromParent(viewGroup);
			viewGroup.LayoutParameters = layoutParams;
			return viewGroup;

		}

		internal static IVisualElementRenderer Convert(Xamarin.Forms.View source, Xamarin.Forms.VisualElement valid)
		{
			IVisualElementRenderer render = (IVisualElementRenderer)source.GetValue(RendererProperty);
			if (render == null)
			{
				render = Platform.CreateRenderer(source);
				source.SetValue(RendererProperty, render);
				var p = PlatformProperty.GetValue(valid);
				PlatformProperty.SetValue(source, p);
				IsPlatformEnabledProperty.SetValue(source, true);
			}
			return render;
		}

		private static Type _platformType = Type.GetType("Xamarin.Forms.Platform.Android.Platform, Xamarin.Forms.Platform.Android", true);

		private static BindableProperty _rendererProperty;
		internal static BindableProperty RendererProperty
		{
			get { return _rendererProperty ?? (_rendererProperty = (BindableProperty)_platformType.GetField("RendererProperty", BindingFlags.NonPublic | BindingFlags.Static | BindingFlags.Public).GetValue(null)); }
		}

		private static PropertyInfo _platform;
		internal static PropertyInfo PlatformProperty
		{
			get
			{
				return _platform ?? (
			  _platform = typeof(VisualElement).GetProperty("Platform", BindingFlags.NonPublic | BindingFlags.Instance));
			}
		}

		private static PropertyInfo _isplatformenabledprop;
		internal static PropertyInfo IsPlatformEnabledProperty
		{
			get
			{
				return _isplatformenabledprop ?? (
			  _isplatformenabledprop = typeof(VisualElement).GetProperty("IsPlatformEnabled", BindingFlags.NonPublic | BindingFlags.Instance));
			}
		}

		private static void removeItemFromParent(ViewGroup item)
		{
			ViewGroup parentItem = (ViewGroup)item.Parent;
			if (parentItem != null)
				parentItem.RemoveView(item);
		}

		internal static ViewGroup ConvertFormsTemplateToNativeView(DataTemplate view,int listindex,int columnIndex, Rectangle size,SfPicker _picker)
		{
			ObservableCollection<object> tempSource = new ObservableCollection<object>();
			if (_picker.ItemsSource != null)
			{
				foreach (var item in _picker.ItemsSource)
				{
					tempSource.Add(item);
				}
			}
			if (tempSource[0] is ObservableCollection<object>)
			{
				for (int i = 0; i < tempSource.Count; i++)
				{
					var source = tempSource[i] as ObservableCollection<object>;
					var templateLayout = view.CreateContent() as Xamarin.Forms.View;
					templateLayout.BindingContext = source[listindex];
					return ConvertFormsToNative(templateLayout, size, _picker);
				}
			}
			else
			{
				var templateLayout = view.CreateContent() as Xamarin.Forms.View;
				templateLayout.BindingContext = tempSource[listindex];
				return ConvertFormsToNative(templateLayout, size, _picker);
			}
			return null;
		}
	}
}